"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImportProcessor = void 0;
const typescript_1 = require("typescript");
const processor_1 = require("./processor");
function isNamedImports(bindings) {
    return bindings.kind === typescript_1.SyntaxKind.NamedImports && Array.isArray(bindings.elements);
}
function isNamespaceImport(bindings) {
    return bindings.kind === typescript_1.SyntaxKind.NamespaceImport;
}
/**
 * Processor for `import module from 'module'`
 */
class ImportProcessor extends processor_1.Processor {
    parse(node) {
        if (this.ts.isImportDeclaration(node) && this.ts.isStringLiteral(node.moduleSpecifier)) {
            if (node.importClause == null || node.getSourceFile() == null) {
                return {
                    node,
                    moduleName: node.moduleSpecifier.text,
                    keepImport: true,
                };
            }
            let typeOnly = node.importClause.isTypeOnly;
            const nonTypeNamedBindings = [];
            let namespaceImport;
            if (!typeOnly && node.importClause.namedBindings) {
                if (isNamedImports(node.importClause.namedBindings)) {
                    node.importClause.namedBindings.elements.forEach(function (importSpecifier) {
                        if (!importSpecifier.isTypeOnly) {
                            nonTypeNamedBindings.push(importSpecifier);
                        }
                    });
                    // In some case nonTypeNamedBindings is empty but this is not a typeOnly import
                    // For example
                    // import a, {type b} from '.'
                    typeOnly = node.importClause.name == null && nonTypeNamedBindings.length === 0;
                }
                else if (isNamespaceImport(node.importClause.namedBindings)) {
                    namespaceImport = node.importClause.namedBindings;
                }
            }
            return {
                node,
                keepImport: false,
                moduleName: node.moduleSpecifier.text,
                typeOnly,
                nonTypeNamedBindings,
                namespaceImport,
            };
        }
        return null;
    }
    updateModuleName(moduleName, { node, keepImport, typeOnly, nonTypeNamedBindings, namespaceImport }) {
        if (keepImport) {
            // For .d.ts file and file without import caluse (for .e.g. import '../src'), preserve the import
            return this.factory.updateImportDeclaration(node, node.modifiers, node.importClause, this.factory.createStringLiteral(moduleName), node.assertClause);
        }
        if (typeOnly) {
            // Do not emit import for type only
            return null;
        }
        return this.factory.updateImportDeclaration(node, node.modifiers, this.factory.updateImportClause(node.importClause, false, node.importClause.name, node.importClause.namedBindings != null && nonTypeNamedBindings.length > 0
            ? this.factory.updateNamedImports(node.importClause.namedBindings, nonTypeNamedBindings)
            : namespaceImport), this.factory.createStringLiteral(moduleName), node.assertClause);
    }
}
exports.ImportProcessor = ImportProcessor;
//# sourceMappingURL=import-processor.js.map